@@ -1,13 +1,14 @@
 use std::fmt;
-use std::fmt::{Debug, Formatter};
+use std::fmt::{Debug, Display, Formatter};
+use std::ops::ControlFlow;
 
 use internal_iterator::{Internal, InternalIterator, IteratorExt};
 use itertools::Itertools;
 use rand::Rng;
 
-use crate::board::{Board, BoardAvailableMoves, Outcome, Player};
+use crate::board::{AllMovesIterator, AvailableMovesIterator, Board, BoardMoves, BoardSymmetry, Outcome, Player};
 use crate::symmetry::D4Symmetry;
-use crate::util::bits::{BitIter, get_nth_set_bit};
+use crate::util::bits::{get_nth_set_bit, BitIter};
 
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub struct Coord(u8);
@@ -101,11 +102,6 @@ impl STTTBoard {
 
 impl Board for STTTBoard {
     type Move = Coord;
-    type Symmetry = D4Symmetry;
-
-    fn can_lose_after_move() -> bool {
-        false
-    }
 
     fn next_player(&self) -> Player {
         self.next_player
@@ -114,8 +110,7 @@ impl Board for STTTBoard {
     fn is_available_move(&self, mv: Self::Move) -> bool {
         assert!(!self.is_done(), "Board must not be done");
 
-        has_bit(self.macro_mask, mv.om()) &&
-            !has_bit(compact_grid(self.grids[mv.om() as usize]), mv.os())
+        has_bit(self.macro_mask, mv.om()) && !has_bit(compact_grid(self.grids[mv.om() as usize]), mv.os())
     }
 
     fn random_available_move(&self, rng: &mut impl Rng) -> Self::Move {
@@ -162,6 +157,14 @@ impl Board for STTTBoard {
         self.outcome
     }
 
+    fn can_lose_after_move() -> bool {
+        false
+    }
+}
+
+impl BoardSymmetry<STTTBoard> for STTTBoard {
+    type Symmetry = D4Symmetry;
+
     fn map(&self, sym: D4Symmetry) -> STTTBoard {
         let mut grids = [0; 9];
         for oo in 0..9 {
@@ -171,7 +174,7 @@ impl Board for STTTBoard {
         STTTBoard {
             grids,
             main_grid: map_grid(sym, self.main_grid),
-            last_move: self.last_move.map(|c| Self::map_move(sym, c)),
+            last_move: self.last_move.map(|c| self.map_move(sym, c)),
             next_player: self.next_player,
             outcome: self.outcome,
             macro_mask: map_grid(sym, self.macro_mask),
@@ -179,31 +182,57 @@ impl Board for STTTBoard {
         }
     }
 
-    fn map_move(sym: D4Symmetry, mv: Coord) -> Coord {
+    fn map_move(&self, sym: D4Symmetry, mv: Coord) -> Coord {
         Coord::from_oo(map_oo(sym, mv.om()), map_oo(sym, mv.os()))
     }
 }
 
-impl<'a> BoardAvailableMoves<'a, STTTBoard> for STTTBoard {
-    type MoveIterator = STTTMoveIterator<'a>;
-    type AllMoveIterator = Internal<CoordIter>;
+impl<'a> BoardMoves<'a, STTTBoard> for STTTBoard {
+    type AllMovesIterator = Internal<CoordIter>;
+    type AvailableMovesIterator = AvailableMovesIterator<'a, STTTBoard>;
 
-    fn all_possible_moves() -> Self::AllMoveIterator {
+    fn all_possible_moves() -> Self::AllMovesIterator {
         Coord::all().into_internal()
     }
 
-    fn available_moves(&'a self) -> Self::MoveIterator {
-        assert!(!self.is_done(), "Board must not be done");
-        STTTMoveIterator { board: self }
+    fn available_moves(&'a self) -> Self::AvailableMovesIterator {
+        assert!(!self.is_done());
+        AvailableMovesIterator(self)
+    }
+}
+
+impl InternalIterator for AllMovesIterator<STTTBoard> {
+    type Item = Coord;
+
+    fn try_for_each<R, F>(self, f: F) -> ControlFlow<R>
+    where
+        F: FnMut(Self::Item) -> ControlFlow<R>,
+    {
+        Coord::all().try_for_each(f)
+    }
+}
+
+impl<'a> InternalIterator for AvailableMovesIterator<'a, STTTBoard> {
+    type Item = Coord;
+
+    fn try_for_each<R, F: FnMut(Self::Item) -> ControlFlow<R>>(self, mut f: F) -> ControlFlow<R> {
+        let board = self.0;
+        for om in BitIter::new(board.macro_mask) {
+            let free_grid = (!compact_grid(board.grids[om as usize])) & STTTBoard::FULL_MASK;
+            for os in BitIter::new(free_grid) {
+                f(Coord::from_oo(om, os))?;
+            }
+        }
+
+        ControlFlow::Continue(())
     }
 }
 
 pub type CoordIter = std::iter::Map<std::ops::Range<u8>, fn(u8) -> Coord>;
 
 impl Coord {
-
     pub fn all() -> CoordIter {
-        (0..81).map(|o| Self::from_o(o))
+        (0..81).map(Self::from_o)
     }
 
     pub fn all_yx() -> CoordIter {
@@ -253,31 +282,13 @@ impl Coord {
 
 impl Debug for Coord {
     fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
-        write!(f, "({}, {})", self.om(), self.os())
+        write!(f, "Coord({}, {})", self.om(), self.os())
     }
 }
 
-//TODO implement a size hint
-//TODO look into other iterator speedup functions that can be implemented
-#[derive(Debug)]
-pub struct STTTMoveIterator<'a> {
-    board: &'a STTTBoard,
-}
-
-impl<'a> InternalIterator for STTTMoveIterator<'a> {
-    type Item = Coord;
-
-    fn find_map<R, F>(self, mut f: F) -> Option<R> where F: FnMut(Self::Item) -> Option<R> {
-        for om in BitIter::new(self.board.macro_mask) {
-            let free_grid = (!compact_grid(self.board.grids[om as usize])) & STTTBoard::FULL_MASK;
-            for os in BitIter::new(free_grid) {
-                if let Some(r) = f(Coord::from_oo(om, os)) {
-                    return Some(r);
-                }
-            }
-        }
-
-        None
+impl Display for Coord {
+    fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
+        write!(f, "({}, {})", self.om(), self.os())
     }
 }
 
@@ -301,10 +312,8 @@ fn is_win_grid(grid: u32) -> bool {
     debug_assert!(has_mask(STTTBoard::FULL_MASK, grid));
 
     const WIN_GRIDS: [u32; 16] = [
-        2155905152, 4286611584, 4210076288, 4293962368,
-        3435954304, 4291592320, 4277971584, 4294748800,
-        2863300736, 4294635760, 4210731648, 4294638320,
-        4008607872, 4294897904, 4294967295, 4294967295
+        2155905152, 4286611584, 4210076288, 4293962368, 3435954304, 4291592320, 4277971584, 4294748800, 2863300736,
+        4294635760, 4210731648, 4294638320, 4008607872, 4294897904, 4294967295, 4294967295,
     ];
     has_bit(WIN_GRIDS[(grid / 32) as usize], (grid % 32) as u8)
 }
@@ -347,7 +356,7 @@ fn symbol_from_tuple(is_available: bool, is_last: bool, player: Option<Player>)
         (false, true, Some(Player::B)) => 'O',
         (true, false, None) => '.',
         (false, false, None) => ' ',
-        _ => unreachable!("Invalid tile state {:?}", tuple)
+        _ => unreachable!("Invalid tile state {:?}", tuple),
     }
 }
 
@@ -359,7 +368,7 @@ fn symbol_to_tuple(c: char) -> (bool, bool, Option<Player>) {
         'O' => (false, true, Some(Player::B)),
         ' ' => (false, false, None),
         '.' => (true, false, None),
-        _ => panic!("unexpected character '{}'", c)
+        _ => panic!("unexpected character '{}'", c),
     }
 }
 
