@@ -10,17 +10,22 @@ fn sttt_empty() {
 
 #[test]
 fn sttt_few() {
-    board_test_main(&board_from_compact_string("                        o  .........               o    x  xxox        x   O  o  "))
+    let board =
+        board_from_compact_string("                        o  .........               o    x  xxox        x   O  o  ");
+    board_test_main(&board)
 }
 
 #[test]
 fn sttt_close() {
-    board_test_main(&board_from_compact_string("x     ooo.Ooo.xx..o  o  oxoxxxoo     x  xo oxx  xo o  x xxooxx  oxox oox  xx xoxo"))
+    let board =
+        board_from_compact_string("x     ooo.Ooo.xx..o  o  oxoxxxoo     x  xo oxx  xo o  x xxooxx  oxox oox  xx xoxo");
+    board_test_main(&board)
 }
 
 #[test]
 fn sttt_draw() {
-    let board = board_from_compact_string("xxx o xo ooXoxxxxoo  o  oxo o xxx   oxoxxoxoxo xooo x oxxoxxoo  xxooxo xxoooxxoxo");
+    let board =
+        board_from_compact_string("xxx o xo ooXoxxxxoo  o  oxo o xxx   oxoxxoxoxo xooo x oxxoxxoo  xxooxo xxoooxxoxo");
     assert_eq!(board.outcome(), Some(Outcome::Draw));
     board_test_main(&board)
-}
\ No newline at end of file
+}
